// -------------------------------------------------------------------------------------
//
// FishFa30.H                                                     Ulrich Mller 19.07.02
//                                                                PCS1700 C++Builder 4.0
//                                                                Version         3.0.23
// FishFa30.Unit : Basisklasse zur Ansteuerung der fischertechnik Interfaces
// aus C++Builder. Die Funktionalitt entspricht der von umFish20Ex.
// Neu sind die Geschwindigkeitssteuerung und der RobMode (ansync. Anfahren
// einer Position).
//
//--------------------------------------------------------------------------------------
#ifndef FishFa30H
#define FishFa30H

// ----------------- struct ftiDCB ------------------------------------------

typedef struct {
    HANDLE hCom;                 // --- PortHandle / Openflag ( = 0)
    DWORD PortID;                //     IO-Adresse bei LPT
    DWORD LPTDelay;              //     LPT Verzgerungsfaktor
    DWORD LPTAnalog;             //     LPT Analogfaktor
    DWORD ECount;                //     Anzahl E-Eingnge (8/16)
	DWORD FID;                   // --- MultiMediaTimer Handle
	DWORD PollInterval;          //     Pollinterval des MM-Timers
	DWORD AnalogScan;            //     Pollen auch der Analogeingnge (0/1)
	DWORD OutputStatus;          // --- akt. Status der M-Ausgnge
    DWORD MotorStatus;           //     Sollstatus der M-Ausgnge (00|01|10)
    DWORD SpeedStatus;           //     den M-Ausgngen zug. Geschwindigkeit
    DWORD ModeStatus;            //     den M-ausgngen zug. Mode
    DWORD intern;
    DWORD InputStatus;           //     Status der E-Eingnge
    DWORD Analogs[2];            //     Werte der A-Eingnge
	DWORD Counters[16];          //     Impulszhler an den E-Eingngen
} ftiDCB;

// --- umFish30 : Funktionsprototypen ------------------------------------------

typedef DWORD (__stdcall *TOpenInterface) (ftiDCB &Interface, LPCSTR ComName);
typedef DWORD (__stdcall *TCloseInterface) (ftiDCB &Interface);
typedef DWORD (__stdcall *TVersion) ();
typedef DWORD (__stdcall *TGetAnalogDirect) (ftiDCB &Interface, DWORD AnalogNr);
typedef BOOL  (__stdcall *TGetInput) (ftiDCB &Interface, DWORD InputNr);
typedef DWORD (__stdcall *TSetLamp) (ftiDCB &Interface, DWORD LampNr, DWORD OnOff);
typedef DWORD (__stdcall *TSetMotor) (ftiDCB &Interface,
        DWORD MotorNr, DWORD Direction);
typedef DWORD (__stdcall *TSetMotorEx) (ftiDCB &Interface,
        DWORD MotorNr, DWORD Direction, DWORD Speed);
typedef DWORD (__stdcall *TSetMotors) (ftiDCB &Interface, DWORD MotorStatus);
typedef DWORD (__stdcall *TSetMotorsEx) (ftiDCB &Interface, DWORD MotorStatus,
        DWORD SpeedStatus);
typedef DWORD (__stdcall *TRobMotors) (ftiDCB &Interface, DWORD MotorStatus,
        DWORD SpeedStatus, DWORD ModeStatus);
typedef DWORD (__stdcall *TRobMotor) (ftiDCB &Interface, DWORD MotorNr,
        DWORD Direction, DWORD Speed, DWORD Counter);
typedef DWORD (__stdcall *TStartDriver) ();
typedef DWORD (__stdcall *TStopDriver) ();

// --- Konstanten / enums ------------------------------------------------------

const DWORD ftiFehler = 0x0001FFFF;

enum Dir{ftiLeft  = 1, ftiRight  = 2, ftiOff = 0, ftiOn  = 1,
         ftiLinks = 1, ftiRechts = 2, ftiAus = 0, ftiEin = 1};
enum Nr {ftiEX  = 0,  ftiEY  = 1,
         ftiE1  = 1,  ftiE2  = 2,  ftiE3  = 3,  ftiE4  = 4,
         ftiE5  = 5,  ftiE6  = 6,  ftiE7  = 7,  ftiE8  = 8,
         ftiE9  = 9,  ftiE10 = 10, ftiE11 = 11, ftiE12 = 12,
         ftiE13 = 13, ftiE14 = 14, ftiE15 = 15, ftiE16 = 16,
         ftiM1  = 1,  ftiM2  = 2,  ftiM3  = 3,  ftiM4  = 4,
         ftiM5  = 5,  ftiM6  = 6,  ftiM7  = 7,  ftiM8  = 8};
enum Speed{ftiNone = 0, ftiHalf = 7, ftiFull    = 15};
enum Wait {ftiEnde = 0, ftiTime = 1, ftiNotHalt = 2,  ftiESC = 3};

class TFishFaceException
{
private:
    int Fnr;
    char* Ftext;
public:
    TFishFaceException(int nr, char* text);
    __property char* Text = {read = Ftext};
    __property int   Nr   = {read = Fnr};
};

//
// --------------------- class TFishFace ---------------------------------------
//

class TFishFace
{

private:

  // ----- umFish30.DLL Daten --------------------------------------------------

    ftiDCB ft;
    HINSTANCE umFishLib;
    BOOLEAN LoadFish(void);
    void FreeFish();
    TOpenInterface   umOpenInterface;
    TCloseInterface  umCloseInterface;
    TVersion         umGetVersion;
    TGetAnalogDirect umGetAnalogDirect;
    TGetInput        umGetInput;
    TSetLamp         umSetLamp;
    TSetMotor        umSetMotor;
    TSetMotorEx      umSetMotorEx;
    TSetMotors       umSetMotors;
    TSetMotorsEx     umSetMotorsEx;
    TRobMotors       umRobMotors;
    TRobMotor        umRobMotor;
    TStartDriver     umStartDriver;
    TStopDriver      umStopDriver;
    bool FNotHalt;
    bool FOpen;
    bool FMessages;

    unsigned int __fastcall GetPollInterval();
    bool         __fastcall GetAnalogScan();
    unsigned int __fastcall GetLPTAnalog();
    unsigned int __fastcall GetLPTDelay();
    bool         __fastcall GetSlave();
    unsigned int __fastcall GetVersion();
    unsigned int __fastcall GetAnalogsEX();
    unsigned int __fastcall GetAnalogsEY();
    unsigned int __fastcall GetInputsP();
    unsigned int __fastcall GetOutputs();

public:

  TFishFace();
  TFishFace(bool AnalogScan, bool Slave, unsigned int PollInterval);
  TFishFace(bool AnalogScan, bool Slave, unsigned int PollInterval,
            unsigned int LPTAnalog, unsigned int LPTDelay);
  ~TFishFace();

    void ClearCounter(unsigned int InputNr);
    void ClearCounters();
    void ClearMotors();
    void CloseInterface();
    bool Finish();
    bool Finish(unsigned int InputNr);
    unsigned int GetAnalog(unsigned int AnalogNr);
    unsigned int GetCounter(unsigned int InputNr);
    bool GetInput(unsigned int InputNr);
    unsigned int GetInputs();
    void OpenInterface(LPCSTR PortName, bool Messages=true);
    void Pause(unsigned int mSek);
    void SetCounter(unsigned int InputNr, unsigned int Value);
    void SetLamp(unsigned int LampNr, bool OnOff);
    void SetMotor(unsigned int MotorNr, unsigned int Direction);
    void SetMotor(unsigned int MotorNr, unsigned int Direction, unsigned int Speed);
    void SetMotor(unsigned int MotorNr, unsigned int Direction,
            unsigned int Speed, unsigned int Counter);
    void SetMotors(unsigned int MotorStatus, unsigned int SpeedStatus,
            unsigned int ModeStatus);
    void SetMotors(unsigned int MotorStatus, unsigned int SpeedStatus);
    void SetMotors(unsigned int MotorStatus);
    void WaitForChange(unsigned int InputNr, unsigned int NrOfChanges,
        unsigned int TermInputNr);
    void WaitForChange(unsigned int InputNr, unsigned int NrOfChanges);
    void WaitForHigh(unsigned int InputNr);
    void WaitForInput(unsigned int InputNr, bool OnOff=true);
    void WaitForLow(unsigned int InputNr);
    unsigned int WaitForMotors(unsigned int mSek, unsigned int M1, unsigned int M2=0,
            unsigned int M3=0, unsigned int M4=0, unsigned int M5=0, unsigned int M6=0,
            unsigned int M7=0, unsigned int M8=0);
    void WaitForPositionDown(unsigned int InputNr, unsigned int& Counter,
            unsigned int Position, unsigned int TermInputNr=0);
    void WaitForPositionUp(unsigned int InputNr, unsigned int& Counter,
            unsigned int Position, unsigned int TermInputNr=0);
    void WaitForTime(unsigned int mSek);
    unsigned int GetAnalogDirect(unsigned int AnalogNr);

    __property unsigned int PollInterval  = { read = GetPollInterval };
    __property bool AnalogScan  = { read = GetAnalogScan };
    __property unsigned int LPTAnalog  = { read = GetLPTAnalog };
    __property unsigned int LPTDelay  = { read = GetLPTDelay };
    __property bool NotHalt  = { read = FNotHalt, write = FNotHalt };
    __property bool Slave  = { read = GetSlave };
    __property unsigned int Version  = { read = GetVersion };
    __property unsigned int AnalogsEX  = { read = GetAnalogsEX };
    __property unsigned int AnalogsEY  = { read = GetAnalogsEY };
    __property unsigned int Inputs  = { read = GetInputsP };
    __property unsigned int Outputs  = { read = GetOutputs };
};

//---------------------------------------------------------------------------
#endif

